using System;
using System.Collections.Generic;
using System.Text;
using System.Threading;

namespace SpiDotNet
{
    /******************************************************************************************
    * This short program demonstrates how to use the .Net wrapper for Compulab's SPI Driver.
    * Evaluation Kit's LCD should be connected to SB-T35, for correct behavior.
    * This program turn the LCD off and than, after 1 sec. on
    ******************************************************************************************/
    class Program
    {
        internal const uint SPI_DEVICE_NUMBER = 4;
        internal const uint TOPOLY_SPI_ADDRESS = 0; //SPI4
        internal const uint LCD_RESET_GPIO = 157;
        internal const int SLEEP_TIME_FOR_RESET = 75;
        internal const uint BACKLIGHT_GPIO = 58;
        internal const uint TD035STEE1_LCD_ID = 0x2008;

        internal const uint MCSPI_PHA_EVEN_EDGES = (1 << 0);
        internal const uint MCSPI_POL_ACTIVELOW = (1 << 1);
        internal const uint MCSPI_POL_ACTIVEHIGH = (0 << 1);
        internal const uint MCSPI_CSPOLARITY_ACTIVELOW = (1 << 6);
        internal const uint MCSPI_CHCONF_TRM_TXRX = (0 << 12);
        internal const uint MCSPI_CHCONF_DMAW_DISABLE = (0 << 14);
        internal const uint MCSPI_CHCONF_DMAR_DISABLE = (0 << 15);
        internal const uint MCSPI_CHCONF_DPE0 = (1 << 16);

        internal static uint MCSPI_CHCONF_CLKD(uint x)
        {
            return ((x & 0x0F) << 2);
        }

        internal static uint MCSPI_CHCONF_WL(uint x)
        {
            return (((x - 1) & 0x1F) << 7);
        }

        private static GpioCmT35 hGPIO;
        private static SpiCmT35 hSPI;

        internal static uint MCSPI_CH3CONF_REG_BASE = MCSPI_PHA_EVEN_EDGES |
                                    MCSPI_POL_ACTIVELOW |
                                    MCSPI_CHCONF_CLKD(8) |
                                    MCSPI_CSPOLARITY_ACTIVELOW |
                                    MCSPI_CHCONF_TRM_TXRX |
                                    MCSPI_CHCONF_DMAW_DISABLE |
                                    MCSPI_CHCONF_DMAR_DISABLE |
                                    MCSPI_CHCONF_DPE0;

        private static void ResetLCD(bool leaveLcdInReset)
        {
            hGPIO.SetAsOutput(LCD_RESET_GPIO, GpioLevel.LOW);
            Thread.Sleep(SLEEP_TIME_FOR_RESET);
            if (!leaveLcdInReset)
            {
                hGPIO.SetAsOutput(LCD_RESET_GPIO, GpioLevel.HIGH);
                Thread.Sleep(SLEEP_TIME_FOR_RESET);
            }
        }

        private static void BacklightEnable(bool enable)
        {
            if (enable)
                hGPIO.SetAsOutput(BACKLIGHT_GPIO, GpioLevel.HIGH);
            else
                hGPIO.SetAsOutput(BACKLIGHT_GPIO, GpioLevel.LOW);
        }

        private static void ConfigureWordLength(uint length, bool isClockActiveHigh)
        {
            uint config = MCSPI_CH3CONF_REG_BASE | MCSPI_CHCONF_WL(length);
            if (isClockActiveHigh)
                config |= MCSPI_POL_ACTIVEHIGH;

            if (!hSPI.SPIConfigure(TOPOLY_SPI_ADDRESS, config))
            {
                throw new InvalidOperationException("Error: Failed to configure SPI" + SPI_DEVICE_NUMBER + ":");
            }
        }

        private static void WriteToLCD(uint uiDataToLcd, ref uint uiDataFromLcd)
        {
            uint[] bin = new uint[1];
            uint[] bout = new uint[1];
            bin[0] = uiDataToLcd;
            bout[0] = uiDataFromLcd;
            if (!hSPI.SPIWriteRead(sizeof(uint), ref bout, bin))
            {
                throw new InvalidOperationException("Error: SPI transaction failed!");
            }
            uiDataFromLcd = bout[0];
            Thread.Sleep(2);
        }

        private static void LCDTurnOn()
        {
            System.Console.WriteLine("TD035::LCDTurnOn");
            uint uiDataFromLcd = 0;

            ResetLCD(false);
            BacklightEnable(true);

            // First 3 commands need to be sent with POL_INACTIVELOW
            ConfigureWordLength(9, true);

            for (int i = 0; i < 3; ++i)
            {
                //Make a transaction on the bus
                WriteToLCD(0x0, ref uiDataFromLcd);
            }


            ConfigureWordLength(18, false);
            /* send 0x17 to 0xB0 */
            WriteToLCD(((0xb0 << 9) + 0x117), ref uiDataFromLcd);
            /* send 0x80 to 0xBC */
            WriteToLCD(((0xbc << 9) + 0x180), ref uiDataFromLcd);
            /* send 0x00 to 0x3B */
            WriteToLCD(((0x3b << 9) + 0x100), ref uiDataFromLcd);
            /* send 0x16 to 0xB0 */
            WriteToLCD(((0xb0 << 9) + 0x116), ref uiDataFromLcd);

            ConfigureWordLength(27, false);
            /* send 0xFF and 0xF9 to 0xB8 */
            WriteToLCD(((0xb8 << 18) + (0x1ff << 9) + 0x1F9), ref uiDataFromLcd);
            ConfigureWordLength(9, false);
            /* send 0x11 */
            WriteToLCD(0x011, ref uiDataFromLcd);

            ConfigureWordLength(18, false);
            /* send 0x01 to 0xBA */
            WriteToLCD(((0xba << 9) + 0x101), ref uiDataFromLcd);
            /* send 0x00 to 0xBB */
            WriteToLCD(((0xbb << 9) + 0x100), ref uiDataFromLcd);
            /* send 0x50 to 0x3A */
            WriteToLCD(((0x3A << 9) + 0x150), ref uiDataFromLcd);
            /* send 0x10 to 0xBF */
            WriteToLCD(((0xbf << 9) + 0x110), ref uiDataFromLcd);
            /* send 0x56 to 0xB1 */
            WriteToLCD(((0xb1 << 9) + 0x156), ref uiDataFromLcd);
            /* send 0x33 to 0xB2 */
            WriteToLCD(((0xb2 << 9) + 0x133), ref uiDataFromLcd);
            /* send 0x11 to 0xB3 */
            WriteToLCD((0xb3 << 9) + 0x111, ref uiDataFromLcd);
            /* send 0x02 to 0xB4 */
            WriteToLCD(((0xb4 << 9) + 0x102), ref uiDataFromLcd);
            WriteToLCD(((0xb6 << 9) + 0x140), ref uiDataFromLcd);
            /* send 0x03 to 0xB7 */
            WriteToLCD(((0xb7 << 9) + 0x103), ref uiDataFromLcd);
            /* send 0x04 to 0xB9 */
            WriteToLCD(((0xb9 << 9) + 0x104), ref uiDataFromLcd);
            /* send 0x00 to 0xBE */
            WriteToLCD(((0xbe << 9) + 0x100), ref uiDataFromLcd);
            /* send 0x11 to 0xC0 */
            WriteToLCD(((0xc0 << 9) + 0x111), ref uiDataFromLcd);
            /* send 0x11 to 0xC1 */
            WriteToLCD(((0xc1 << 9) + 0x111), ref uiDataFromLcd);
            /* send 0x11 to 0xC2 */
            WriteToLCD(((0xc2 << 9) + 0x111), ref uiDataFromLcd);

            ConfigureWordLength(27, false);

            /* send 0x20 and 0x40 to 0xC3 */
            WriteToLCD(((0xc3 << 18) + (0x120 << 9) + 0x140), ref uiDataFromLcd);
            /* send 0x60 and 0xC0 to 0xC4 */
            WriteToLCD(((0xc4 << 18) + (0x160 << 9) + 0x1C0), ref uiDataFromLcd);
            /* send 0x10 and 0x20 to 0xC5 */
            WriteToLCD(((0xc5 << 18) + (0x110 << 9) + 0x120), ref uiDataFromLcd);
            /* send 0x60 and 0xC0 to 0xC6 */
            WriteToLCD(((0xc6 << 18) + (0x160 << 9) + 0x1C0), ref uiDataFromLcd);
            /*send 0x02 and 0x68 to 0xEC */
            WriteToLCD(((0xec << 18) + (0x102 << 9) + 0x168), ref uiDataFromLcd);

            ConfigureWordLength(18, false);
            /* send 0x02 to 0xCF */
            WriteToLCD(((0xcf << 9) + 0x102), ref uiDataFromLcd);

            ConfigureWordLength(27, false);
            /*send 0x08 and 0x04 to 0xD0  */
            WriteToLCD(((0xd0 << 18) + (0x108 << 9) + 0x104), ref uiDataFromLcd);

            ConfigureWordLength(18, false);
            /* send 0x01 to 0xD1 */
            WriteToLCD(((0xd1 << 9) + 0x101), ref uiDataFromLcd);

            ConfigureWordLength(27, false);
            /* send 0x14 and 0x00 to 0xD2 */
            WriteToLCD(((0xd2 << 18) + (0x114 << 9) + 0x100), ref uiDataFromLcd);
            /* send 0x1A and 0x0F to 0xD3 */
            WriteToLCD(((0xd3 << 18) + (0x11a << 9) + 0x10f), ref uiDataFromLcd);
            /* send 0x1F and 0xAF to 0xD4 */
            WriteToLCD(((0xd4 << 18) + (0x11f << 9) + 0x1af), ref uiDataFromLcd);

            ConfigureWordLength(18, false);
            /* send 0x14 to 0xD5 */
            WriteToLCD(((0xd5 << 9) + 0x114), ref uiDataFromLcd);

            ConfigureWordLength(9, false);

            Thread.Sleep(40);
            //Send DISPLAY ON command
            WriteToLCD(0x029, ref uiDataFromLcd);

            //Read the LCD ID here
            ConfigureWordLength(32, false);
            WriteToLCD(0x02000000, ref uiDataFromLcd);

            if ((uiDataFromLcd & 0xFFFF) == TD035STEE1_LCD_ID)
            {
                //After this startup sequence it takes the LCD several seconds to show an image
                System.Console.WriteLine("TD035STEE1: Turned display on.");
            }
            else
            {

                throw new InvalidOperationException(("TD035STEE1: ERROR. LCD id = " + (uiDataFromLcd & 0xFFFF) + "."));
            }

            System.Console.WriteLine("TD035:: Finished LCD VGA init.");
        }

        private static void LCDTurnOff()
        {
            uint uiDataFromLCD = 0;
            ConfigureWordLength(9, false);
            //Turn Display off
            WriteToLCD(0x28, ref uiDataFromLCD);
            ResetLCD(true);
            BacklightEnable(false);
        }

        static void Main(string[] args)
        {
            hGPIO = new GpioCmT35();
            hSPI = new SpiCmT35(SPI_DEVICE_NUMBER);
            LCDTurnOff();
            Thread.Sleep(1000);
            LCDTurnOn();
        }
    }
}
